import type { Model, ModelIndexer, ModelProperty, RekeyableMap, SourceModel, Type } from "../../../core/types.js";
import { DecoratorArgs } from "../utils.js";
/**
 * A descriptor for creating a model.
 * @experimental
 */
export interface ModelDescriptor {
    /**
     * The name of the Model. If name is provided, it is a Model  declaration.
     * Otherwise, it is a Model expression.
     */
    name?: string;
    /**
     * Decorators to apply to the Model.
     */
    decorators?: DecoratorArgs[];
    /**
     * Properties of the model.
     */
    properties: Record<string, ModelProperty>;
    /**
     * Models that extend this model.
     */
    derivedModels?: Model[];
    /**
     * Models that this model extends.
     */
    sourceModels?: SourceModel[];
    /**
     * The indexer property of the model.
     */
    indexer?: ModelIndexer;
}
/**
 * Utilities for working with models.
 * @experimental
 */
export interface ModelKit {
    /**
     * Create a model type.
     *
     * @param desc The descriptor of the model.
     */
    create(desc: ModelDescriptor): Model;
    /**
     * Check if the given `type` is a model..
     *
     * @param type The type to check.
     */
    is(type: Type): type is Model;
    /**
     * Check this is an anonyous model. Specifically, this checks if the
     * model has a name.
     *
     * @param type The model to check.
     */
    isExpresion(type: Model): boolean;
    /**
     * If the input is anonymous (or the provided filter removes properties)
     * and there exists a named model with the same set of properties
     * (ignoring filtered properties), then return that named model.
     * Otherwise, return the input unchanged.
     *
     * This can be used by emitters to find a better name for a set of
     * properties after filtering. For example, given `{ @metadata prop:
     * string} & SomeName`, and an emitter that wishes to discard properties
     * marked with `@metadata`, the emitter can use this to recover that the
     * best name for the remaining properties is `SomeName`.
     *
     * @param model The input model
     * @param filter An optional filter to apply to the input model's
     * properties.
     */
    getEffectiveModel(model: Model, filter?: (property: ModelProperty) => boolean): Model;
    /**
     * Given a model, return the type that is spread
     * @returns the type that is spread or undefined if no spread
     */
    getSpreadType: (model: Model) => Type | undefined;
    /**
     * Gets all properties from a model, explicitly defined and implicitly defined.
     * @param model model to get the properties from
     */
    getProperties(model: Model, options?: {
        includeExtended?: boolean;
    }): RekeyableMap<string, ModelProperty>;
}
interface TypekitExtension {
    /**
     * Utilities for working with models.
     * @experimental
     */
    model: ModelKit;
}
declare module "../define-kit.js" {
    interface Typekit extends TypekitExtension {
    }
}
export {};
//# sourceMappingURL=model.d.ts.map